#!/bin/bash -e

# Function to install basic packages
install_basic_packages() {
    echo "================= Installing basic packages ================"
    echo "Recieved Input as $IMAGE_TYPE"

    if [[ $IMAGE_TYPE == "RUNTIME" ]]; then
        echo "================= Adding some global settings ==================="
        mkdir -p "$HOME/.ssh/"
        echo -e "Host *\n    StrictHostKeyChecking no" > "$HOME/.ssh/config"
        echo 'APT::Get::Assume-Yes "true";' > "90forceyes"
        mv "90forceyes" /etc/apt/apt.conf.d/
        touch "$HOME/.ssh/known_hosts"
        mkdir -p /etc/drydock
        export architecture=$(dpkg --print-architecture)
        apt-get update
        apt-get install ca-certificates
        apt-get install -y sudo
    else
        # Remove any broken lists; lists will be repopulated with apt-get update
        sudo rm -rf /var/lib/apt/lists/*

        export DEBIAN_FRONTEND=noninteractive
        export PIP_ROOT_USER_ACTION=ignore
        export architecture=$(dpkg --print-architecture)

        echo 'debconf debconf/frontend select Noninteractive' | sudo debconf-set-selections
        echo "================= Done adding neccessary variables ==================="
        sudo apt-get update
        sudo apt-get install ca-certificates
        sudo apt-get remove -y unattended-upgrades
    fi


    sudo apt-get install -y \
                curl \
                gettext \
                ftp \
                groff \
                mercurial \
                openssh-client \
                smbclient \
                software-properties-common \
                sudo \
                wget \
                unzip \
                apt-transport-https \
                ca-certificates \
                gnupg \
                lsb-release \
                python3 \
                python3-pip \
                python3-software-properties \
                python3-dev

    python3_version=$(python3 --version 2>&1 | cut -d' ' -f2)
    echo "\"python3\": \"$python3_version\"," >> "$VERSION_FILE"
}

# Function to install Python packages
install_python_packages() {
    echo "================= Installing Python packages =================="
    sudo pip3 install virtualenv
    sudo pip3 install cryptography
    sudo pip3 install -q pyOpenSSL
}

# Function to add JQ
install_jq() {
    echo "================= Adding JQ========================="
    sudo apt-get install -y -q jq
    jq_version=$(jq --version | cut -d'-' -f2)
    echo "\"jq\": \"$jq_version\"," >> "$VERSION_FILE"
}

# Function to install Git
install_git() {
    echo "================= Installing Git===================="
    sudo add-apt-repository ppa:git-core/ppa -y
    sudo apt-get update -qq
    sudo apt-get install -y -q git
    git_version=$(git --version | cut -d' ' -f3)
    echo "\"git\": \"$git_version\"," >> "$VERSION_FILE"
}

# Function to add Google Cloud SDK
install_gcloud() {
    echo "================= Adding gcloud  ============"
    echo "deb [signed-by=/usr/share/keyrings/cloud.google.asc] https://packages.cloud.google.com/apt cloud-sdk main" | sudo tee -a /etc/apt/sources.list.d/google-cloud-sdk.list
    curl https://packages.cloud.google.com/apt/doc/apt-key.gpg | sudo tee /usr/share/keyrings/cloud.google.asc
    sudo apt-get update && sudo apt-get install google-cloud-cli
    gcloud_version=$(gcloud --version | head -n1 | cut -d' ' -f4)
    echo "\"gcloud\": \"$gcloud_version\"," >> "$VERSION_FILE"
}

# Function to add AWS CLI
install_awscli() {
    echo "================ Adding awscli===================="
    if [[ $architecture == "amd64" ]]; then
        curl "https://awscli.amazonaws.com/awscli-exe-linux-x86_64.zip" -o "awscliv2.zip"
    else 
        curl "https://awscli.amazonaws.com/awscli-exe-linux-aarch64.zip" -o "awscliv2.zip"
    fi
    unzip awscliv2.zip && sudo ./aws/install

    awscli_version=$(aws --version 2>&1 | cut -d' ' -f1 | cut -d'/' -f2)
    echo "\"awscli\": \"$awscli_version\"," >> "$VERSION_FILE"
}

install_azure_cli() {
    curl -sL https://aka.ms/InstallAzureCLIDeb | sudo bash
    azure_cli_version=$(az --version 2>&1 | awk '/azure-cli/ {print $2}' | head -n1)
}

# Function to install Helm 2
install_helm2() {
    HELM_VERSION=2.16.9
    echo "================= helm 2 cli $HELM_VERSION ======================================"
    if [[ $architecture == "amd64" ]]; then
        curl -sSL -o helm.tar.gz https://get.helm.sh/helm-v"$HELM_VERSION"-linux-amd64.tar.gz
        tar xzvf helm.tar.gz
        chmod +x ./linux-amd64/helm
        chmod +x ./linux-amd64/tiller
        sudo mv ./linux-amd64/helm /usr/local/bin/helm2
        sudo mv ./linux-amd64/tiller /usr/local/bin/tiller
        rm -fr ./linux-amd64
        rm -f helm.tar.gz
    else 
        curl -sSL -o helm.tar.gz https://get.helm.sh/helm-v"$HELM_VERSION"-linux-arm64.tar.gz
        tar xzvf helm.tar.gz
        chmod +x ./linux-arm64/helm
        chmod +x ./linux-arm64/tiller
        sudo mv ./linux-arm64/helm /usr/local/bin/helm2
        sudo mv ./linux-arm64/tiller /usr/local/bin/tiller
        rm -fr ./linux-arm64
        rm -f helm.tar.gz
    fi

    echo "\"helm2\": \"$HELM_VERSION\"," >> "$VERSION_FILE"
}

# Function to install Helm 3
install_helm3() {
    echo "================= helm 3 cli======================================"
    curl https://baltocdn.com/helm/signing.asc | gpg --dearmor | sudo tee /usr/share/keyrings/helm.gpg > /dev/null
    echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/helm.gpg] https://baltocdn.com/helm/stable/debian/ all main" | sudo tee /etc/apt/sources.list.d/helm-stable-debian.list
    sudo apt-get update && sudo apt-get install helm

    helm3_version=$(helm version --short | cut -d':' -f2)
    echo "\"helm3\": \"$helm3_version\"," >> "$VERSION_FILE"
}

install_docker() {
    if [[ $IMAGE_TYPE == "RUNTIME" ]]; then
        echo "RUNTIME IMAGE - skipping creating rootless user"
    else
        export USERNAME="pipelinesRootless"
        export GROUPNAME="pipelinesRootless"
        export JFROG_DIR="/home/$USERNAME/.jfrog/pipelines"
        export DOCKER_VERSION="20.10.22"

        if [ -d "/home/$USERNAME" ]; then
        echo "Removing /home/$USERNAME"
        rm -rf /home/$USERNAME
        fi

        sudo groupadd -g 1066 $GROUPNAME
        sudo adduser --system --home /home/$USERNAME --gid 1066 --uid 1066 --shell /bin/bash $USERNAME

        sudo mkdir -p $JFROG_DIR
        sudo mkdir -p /home/$USERNAME/.local/share/docker
        sudo mkdir -p /home/$USERNAME/.config/systemd/user
        sudo mkdir -p $JFROG_DIR/var/etc/reqKick

        sudo chown -R $GROUPNAME:$USERNAME /home/$USERNAME

        sudo loginctl enable-linger $USERNAME
        sudo apt-get install -y uidmap

        export XDG_RUNTIME_DIR=/run/user/1066
        export HOME=/home/$USERNAME

        export PATH=/home/$USERNAME/bin:$PATH
        export DOCKER_HOST=unix://$XDG_RUNTIME_DIR/docker.sock

    fi

    sudo install -m 0755 -d /etc/apt/keyrings
    curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/docker.gpg && sudo chmod a+r /etc/apt/keyrings/docker.gpg

    # Add the repository to Apt sources:
    echo \
    "deb [arch="$(dpkg --print-architecture)" signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu \
    "$(. /etc/os-release && echo "$VERSION_CODENAME")" stable" | \
    sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
    sudo apt-get update && sudo apt-get install -y docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose-plugin
}


install_gitlfs(){
    echo "================= Installing Gitlfs  ===================="
    curl -s https://packagecloud.io/install/repositories/github/git-lfs/script.deb.sh > gitlfs-script.sh
    sed -i 's|curl -sSf "${apt_config_url}"|curl -sSLf "${apt_config_url}"|' gitlfs-script.sh || echo "SED failed, continuing trying to install"
    sudo apt-get install git-lfs
}

install_jfrog_cli_v2(){
    JFROG_VERSION=$1
    echo "================= Adding jfrog-cli $JFROG_VERSION  ================"
    if [[ $architecture == "amd64" ]]; then
        wget -nv https://releases.jfrog.io/artifactory/jfrog-cli/v2-jf/"$JFROG_VERSION"/jfrog-cli-linux-amd64/jf -O jfrog
    elif [[ $architecture == "arm64" ]]; then
        wget -nv https://releases.jfrog.io/artifactory/jfrog-cli/v2-jf/"$JFROG_VERSION"/jfrog-cli-linux-arm64/jf -O jfrog
    fi
    sudo chmod +x jfrog
    sudo mv jfrog /usr/bin/jfrog
}

install_kubectl(){
    KUBECTL_VERSION=$1
    echo "================= Adding kubectl ================"
    curl -fsSL https://pkgs.k8s.io/core:/stable:/v$KUBECTL_VERSION/deb/Release.key | sudo gpg --dearmor -o /etc/apt/keyrings/kubernetes-apt-keyring.gpg
    echo "deb [signed-by=/etc/apt/keyrings/kubernetes-apt-keyring.gpg] https://pkgs.k8s.io/core:/stable:/v$KUBECTL_VERSION/deb/ /" | sudo tee /etc/apt/sources.list.d/kubernetes.list
    sudo apt-get update && sudo apt-get install -y kubectl
}

add_hashicorp(){
    sudo apt-get update && sudo apt-get install -y gnupg software-properties-common
    wget -O- https://apt.releases.hashicorp.com/gpg | \
                gpg --dearmor | \
                sudo tee /usr/share/keyrings/hashicorp-archive-keyring.gpg

    gpg --no-default-keyring \
        --keyring /usr/share/keyrings/hashicorp-archive-keyring.gpg \
        --fingerprint

    echo "deb [signed-by=/usr/share/keyrings/hashicorp-archive-keyring.gpg] \
            https://apt.releases.hashicorp.com $(lsb_release -cs) main" | \
            sudo tee /etc/apt/sources.list.d/hashicorp.list
}

install_terraform(){
    echo "================ Adding terraform ===================="
    add_hashicorp
    sudo apt update && apt-get install terraform
}

install_packer(){
    echo "================ Adding packer ===================="
    add_hashicorp
    sudo apt update && apt-get install packer
}

install_gcc(){
    GCC_VERSION=$1
	echo "======================= Installing gcc======================"
	add-apt-repository -y ppa:ubuntu-toolchain-r/test
	apt-get update
	apt-get install gcc-$GCC_VERSION g++-$GCC_VERSION
	update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-$GCC_VERSION 90 --slave /usr/bin/g++ g++ /usr/bin/g++-$GCC_VERSION
	update-alternatives --install /usr/bin/gcov gcov /usr/bin/gcov-$GCC_VERSION 90
	gcc --version
}

install_cpp_packages(){
	echo "=================== Install packages for cpp ======================"

	apt-get install && apt-get install \
	autoconf \
	automake \
	ccache \
	libssl-dev

}

install_clang(){
    CLANG_VERSION=$1
	echo "==================== Installing clang==============="
	apt-get update && apt-get -y install clang-$CLANG_VERSION
	update-alternatives --install /usr/bin/clang clang /usr/bin/clang-$CLANG_VERSION 90 --slave /usr/bin/clang++ clang++ /usr/bin/clang++-$CLANG_VERSION
	clang --version
}

install_dotnet(){
    DOTNET_SDK_VERISON=$1
	echo "===================Installing Dotnet============================="
    # apt-get update && apt-get install -y apt-transport-https
	# apt-get update && \
	# sudo apt-get install -y dotnet-sdk-$DOTNET_SDK_VERISON
    wget https://dot.net/v1/dotnet-install.sh -O dotnet-install.sh
    chmod +x ./dotnet-install.sh
    ./dotnet-install.sh --channel $DOTNET_SDK_VERISON --install-dir /usr/share/dotnet
}

install_go(){
    export GOVER=$1
	echo "================= Install packages for Go ==================="
	apt-get update && apt-get install \
		autotools-dev \
		autoconf \
		bison \
		cmake \
		scons \
		binutils \
		gcc \
		bsdmainutils \
		bzr

	echo "================= Install gvm ==================="
	curl -s -L https://raw.githubusercontent.com/moovweb/gvm/master/binscripts/gvm-installer | bash

	export CGO_ENABLED=0
	[[ -s "$HOME/.gvm/scripts/gvm" ]] && source $HOME/.gvm/scripts/gvm || echo "Could not source from $HOME/.gvm/scripts/gvm"
	export CGO_ENABLED=0

	# Install gvm
	echo "================= Install Go==================="
	export GOROOT_BOOTSTRAP=$GOROOT
	gvm install go$GOVER --prefer-binary
	source /root/.gvm/scripts/gvm || echo "Could not source from /root/.gvm/scripts/gvm"
	gvm use go$GOVER

	echo "======= Install go linter ========="
	curl -sSfL https://raw.githubusercontent.com/golangci/golangci-lint/master/install.sh | sh -s -- -b $(go env GOPATH)/bin
	golangci-lint --version
}


install_gradle(){
    GRADLE_VERSION=$1
	apt-get update
	dpkg --purge --force-depends ca-certificates-java
	apt-get install ca-certificates-java

	echo "================ Installing gradle "$GRADLE_VERSION"  ================="
	wget -nv https://services.gradle.org/distributions/gradle-$GRADLE_VERSION-all.zip
	unzip -qq gradle-$GRADLE_VERSION-all.zip -d /usr/local && rm -f gradle-$GRADLE_VERSION-all.zip
	ln -fs /usr/local/gradle-$GRADLE_VERSION/bin/gradle /usr/bin
	echo "export PATH=$PATH:/usr/local/gradle-$GRADLE_VERSION/bin" >> /etc/drydock/.env
}

install_apache_maven(){
	APACHE_MAVEN_VERSION=$1
	echo "================ Installing apache-maven "$APACHE_MAVEN_VERSION" ================="
	apt-get update
	wget -nv https://dlcdn.apache.org/maven/maven-3/$APACHE_MAVEN_VERSION/binaries/apache-maven-$APACHE_MAVEN_VERSION-bin.tar.gz
	tar xzf apache-maven-$APACHE_MAVEN_VERSION-bin.tar.gz -C /usr/local && rm -f apache-maven-$APACHE_MAVEN_VERSION-bin.tar.gz
	ln -fs /usr/local/apache-maven-$APACHE_MAVEN_VERSION/bin/mvn /usr/bin
	echo "export PATH=$PATH:/usr/local/apache-maven-$APACHE_MAVEN_VERSION/bin" >> /etc/drydock/.env
	echo "export M2_HOME=/usr/local/apache-maven-$APACHE_MAVEN_VERSION/" >> /etc/drydock/.env
}

install_apache_ant(){
	APACHE_ANT_VERSION=$1
	echo "================ Installing apache-ant "$APACHE_ANT_VERSION" ================="
	apt-get update
	wget -nv https://archive.apache.org/dist/ant/binaries/apache-ant-$APACHE_ANT_VERSION-bin.tar.gz
	tar xzf apache-ant-$APACHE_ANT_VERSION-bin.tar.gz -C /usr/local && rm -f apache-ant-$APACHE_ANT_VERSION-bin.tar.gz
	ln -fs /usr/local/apache-ant-$APACHE_ANT_VERSION/bin/ant /usr/bin
	echo "export ANT_HOME=/usr/local/apache-ant-$APACHE_ANT_VERSION" >> /etc/drydock/.env
	echo "export PATH=$PATH:/usr/local/apache-ant-$APACHE_ANT_VERSION/bin" >> /etc/drydock/.env
}

install_jdk(){
    JDK_VERSION=$1
	apt-get install -y openjdk-$JDK_VERSION-jdk
}

install_nvm(){
	export NVM_VERSION=$1
	echo "================= Installing NVM $NVM_VERSION ================"
	curl -sS https://raw.githubusercontent.com/creationix/nvm/v"$NVM_VERSION"/install.sh | bash
}

install_nodejs(){
    NODE_VERSION=$1
	echo "=============== Installing Node $NODE_VERSION ============="
	. /root/.nvm/nvm.sh && nvm install $NODE_VERSION
}
